/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

package com.topcoder.rules.dependencymap.view.stresstests;

import junit.framework.TestCase;
import com.topcoder.rules.dependencymap.view.*;
import java.util.*;
import java.io.File;


/**
*   Stress test class for the DependencyRenderer class.
*/
public class StressSVGRenderer extends TestCase
{
    /**
    * Sample dependency. As per the component specification
    */
    Dependency phoneticSpellCheck;

    /**
    * Dependency purposely built during setup to have
    * a large number of sub-elements
    */
    Dependency myDependency;

    /**
    * number of renderers used for testing
    */
    private final int maxNumRenderers = 100;

    /**
    * number of levels the test dependency tree will have
    */
    private final int maxNumLevels = 4;

    /**
    * number of sub-elements for each level of the test dependency
     * this number is even so as to avoid the blank column issue
    */
    private final int maxNumElements = 11;

    /**
    * List of Diagram renderers
    */
    private List liste = new ArrayList();

    /**
    * Template name for the test files
    */
    private final String testFilePath = "../../test_files/file";

    /**
    * counter for number of dependants
    */
    private int totalCount;

    /**
     *
     */
    public void setUp()
    {
        //create the sample dependency graph
        Dependency configManager = new Dependency("Config Manager");
        Dependency spellCheck = new Dependency("Spell Check");
        spellCheck.addDependant(configManager);
        Dependency phoneticPatternMatching = new Dependency("Phonetic Pattern Matching");
        phoneticSpellCheck = new Dependency("Phonetic Spell Check");
        phoneticSpellCheck.addDependant(configManager);
        phoneticSpellCheck.addDependant(spellCheck);
        phoneticSpellCheck.addDependant(phoneticPatternMatching);

        //creates the renderers and add them to the list
        for(int i = 0; i < maxNumRenderers; ++i)
        {
            String fileName = testFilePath + i;
            liste.add(new SVGRenderer(new File(fileName)));
        }

        //create the large dependency graph
        totalCount = 1;
        myDependency = new Dependency("largeDependency");
        Vector currentLevel = new Stack();
        currentLevel.add(myDependency);
        for(int i = 0; i < maxNumLevels; ++i)
        {
            Vector nextLevel = new Stack();
            while(!currentLevel.isEmpty())
            {
                Dependency current = (Dependency)currentLevel.firstElement();
                currentLevel.removeElementAt(0);
                for(int j = 0; j < maxNumElements; ++j)
                {
                    Dependency d = new Dependency("level " + i + " element " + j);
                    current.addDependant(d);
                    nextLevel.add(d);
                    ++totalCount;
                }
            }
            currentLevel = new Vector(nextLevel);
        }
    }

    /**
    * Removes the temporary files
    */
    public void tearDown()
    {
        for(int i = 0; i < maxNumRenderers; ++i)
        {
            String fileName = testFilePath + i;
            File f = new File(fileName);
            f.delete();
        }
    }

    /**
    * Attempts to render a dependency graph
    * with a large number of renderers
    */
    public void testManyRenderers() throws RendererException
    {
        // Create the Dependency Renderer and add the diagram renderers
        DependencyRenderer renderer = new DependencyRenderer();
        for(int i = 0; i < maxNumRenderers; ++i)
        {
            renderer.addDiagramRenderer((DiagramRenderer)liste.get(i));
        }
        long startTime = new Date().getTime();
        // Render the dependency
        renderer.render(phoneticSpellCheck);
        long ts = new Date().getTime() - startTime;
        System.out.println("Rendering to " + maxNumRenderers + " renderers took " + ts + " ms");
    }

    /**
    * Attempts to render a large dependency graph
    */
    public void testLargeGraph() throws RendererException
    {
        // Create the Dependency Renderer and add a diagram renderer
        DependencyRenderer renderer = new DependencyRenderer();
        renderer.addDiagramRenderer((DiagramRenderer)liste.get(0));
        long startTime = new Date().getTime();
        // Render the dependency
        renderer.render(myDependency);
        long ts = new Date().getTime() - startTime;
        System.out.println("Rendering a graph with " + maxNumLevels + " levels and " + maxNumElements + " elements per level  (total of " + totalCount + " elements) took " + ts + " ms");
    }
}
